/***************************************************************************
                          ccmdprocessor.cpp  -  command processor
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : P Jul 5 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include "ccmdprocessor.h"

#include "caliaslist.h"
#include "cansiparser.h"
#include "cconnection.h"
#include "cconnprefs.h"
#include "cconsole.h"
#include "cpluginmanager.h"
#include "csession.h"
#include "cscriptlist.h"
#include "ctimerlist.h"
#include "cvariablelist.h"
#include "cgrouplist.h"

#include "kmuddy.h"

#include <klocale.h>

extern KMuddy * kmuddy;
using namespace std;

cCmdProcessor::cCmdProcessor ()
{
  separstr = ";";
  walkstr = ".";
  scriptstr = "/";
  multistr = "#";
  focusstr = ":";
  noparsestr = "'";
  
  allowemptywalkstr = false;
  stripSpaces = false;
  parsing = true;
  
  notifications.clear();
  
}

cCmdProcessor::~cCmdProcessor()
{

}

void cCmdProcessor::setParsing (bool value)
{
  parsing = value;
}

void cCmdProcessor::setCmdSeparatorString (QString str)
{
  separstr = str.stripWhiteSpace();
}

void cCmdProcessor::setSpeedWalkString (QString str)
{
  walkstr = str.stripWhiteSpace();
}

void cCmdProcessor::setScriptString (QString str)
{
  scriptstr = str.stripWhiteSpace();
}

void cCmdProcessor::setMultiCommandString (QString str)
{
  multistr = str.stripWhiteSpace();
}

void cCmdProcessor::setFocusCommandString (QString str)
{
  focusstr = str.stripWhiteSpace();
}

void cCmdProcessor::setNoParseString (QString str)
{
  noparsestr = str.stripWhiteSpace();
}

void cCmdProcessor::setAllowEmptyWalkStr (bool val)
{
  allowemptywalkstr = val;
}

bool cCmdProcessor::willExpandBS (bool isAliasTrigger)
{
  bool expbs;
  //shall we expand backslashes?
  if (expandbackslashes)
    expbs = backslashAT ? isAliasTrigger : true;
  else
    expbs = false;
  return expbs;
}

bool cCmdProcessor::getNext (QString &from, QString &command, bool isAliasTrigger)
//get next command from a list of commands separated by separstr-s
//this class also handles \n, other backslashes are kept intact
{
  if (from.length() == 0)
    return false;

  //will we expand backslashes?
  bool expbs = willExpandBS (isAliasTrigger);
  
  //part of the string before the first occurence of separstr
  QString separatedPart;
  if (separstr.length() > 0)
  {
    int pos = from.find (separstr);
    separatedPart = (pos == -1) ? from : from.left (pos);
  }
  else
    separatedPart = from;
  
  int len = separatedPart.length();

  //if we expand backslashes, then we may also encounter the \n string
  int nnpos = -1;
  if (expbs)
  {
    //We cannot use find, because we must expand \n but NOT \\n
    //nnpos = separatedPart.find ("\\n");
    bool bs = false;
    for (int i = 0; i < len; i++)
      if (bs)
      {
        if (separatedPart[i] == 'n')  //found \n !
        {
          nnpos = i - 1;
          break;
        }
        bs = false;
      }
      else
        if (separatedPart[i] == '\\')
          bs = true;    
  }
  if (nnpos == -1)
  {
    command = from.left (len);
    from.remove (0, len + separstr.length());
  }
  else
  {
    command = from.left (nnpos);
    //length of \n is 2
    from.remove (0, nnpos + 2);
  }
  //strip whitespaces from that command, if needed!
  if (stripSpaces)
    command = command.stripWhiteSpace ();
  return true;
}

int cCmdProcessor::isSpeedWalkCommand (QString command)
{
  bool sw = false;
  
  if ((walkstr.length() > 0) && (command.startsWith (walkstr)))
    sw = true;

  //empty walk-string (if allowed)
  if ((walkstr.length() == 0) && allowemptywalkstr)
    sw = true;
  
  int pos = -1;
  if (sw)
  {
    pos = walkstr.length();
    for (unsigned int i = pos; i < command.length(); i++)
      switch (command[i].latin1()) {
        case ' ':   //spaces are ignored
        case 'n':
        case 'e':
        case 's':
        case 'w':
        case 'j':
        case 'k':
        case 'l':
        case 'm':
        case 'u':
        case 'd':
        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9': break;
        default: return -1;
      };
  }
  else
    return -1;

  //passed all the tests - it's speed-walk command!
  return pos;
}

unsigned int cCmdProcessor::repeater (QString &command)
{
  bool multi = false;
  if ((multistr.length() > 0) && (command.startsWith (multistr)))
    multi = true;
  
  if (multi)
  //chance is that there is some repeater
  {
    QString cmd = command;
    cmd.remove (0, multistr.length());
    cmd = cmd.stripWhiteSpace ();
    QString rep = cmd.section (' ', 0, 0); //first section
    //if rep is a number, then this is a multi-command
    bool ok;
    unsigned int repc = rep.toUInt (&ok);
    if (ok) //GREAT!!! This really is a repeater command!!!
    {
      command = command.section (' ', 1); //everything except the 1st section

      //limit repeat count if needed
      cConnPrefs *cp = sess->connection()->getConnPrefs();
      bool limitRepeats = ((cp == 0) ? true : cp->limitRepeater());

      if ((repc > 100) && limitRepeats)
      {
        QString exclaim = i18n ("Sorry, count is limited to 100.\n");
        if (cp != 0)
          exclaim += i18n ("You can disable this limit in Profile / MUD Preferences.");
        sess->console()->systemMessage (exclaim);
        //return 0 - nothing will be sent...
        return 0;
      }
      return repc;
    }
    else       //no repeater here...
      return 1;
  }
  else  //no repeater -> count is 1
    return 1;
}

void cCmdProcessor::fixBackslashes (QString &command, bool isAliasTrigger)
{
  //do nothing if we don't want to expand here
  if (!(willExpandBS (isAliasTrigger)))
    return;

  QString cmd = "";
  
  bool backslash = false;
  int len = command.length();
  for (int i = 0; i < len; i++)
  {
    QChar ch = command[i];
    if (backslash)
    {
      if (ch.latin1() == 't')  //got \t
        cmd += (QChar) '\t';
      else if (ch.latin1() == 'e')  //got \e
        cmd += (QChar) '\e';
      else
        cmd += ch;
      backslash = false;
    }
    else
    {
      if (ch == QChar ('\\'))
        backslash = true;
      else
        cmd += command[i];
    }
  }
  command = cmd;
}

void cCmdProcessor::protectBackslashes (QString &command, bool isAliasTrigger)
{
  //only protect if: this isn't in an alias/trigger, BS expansion in AT only
  if (expandbackslashes && backslashAT && (!isAliasTrigger))
  {
    //will substitute all BSs for characters 0x01
    QString cmd;
    int len = command.length();
    for (int i = 0; i < len; i++)
      if (command[i].latin1() != '\\')
        cmd += command[i];
      else
        cmd += QChar ('\x01');
    command = cmd;
  }
}

void cCmdProcessor::unprotectBackslashes (QString &command, bool)
{
  //only unprotect if: BS expansion in AT only
  if (expandbackslashes && backslashAT)
  {
    //will substitute all BSs for characters 0x01
    QString cmd;
    int len = command.length();
    for (int i = 0; i < len; i++)
      if (command[i].latin1() != '\x01')
        cmd += command[i];
      else
        cmd += QChar ('\\');
    command = cmd;
  }
}

void cCmdProcessor::processSpeedWalk (const QString &command, int pos)
{
  //we could have computed the position again, but it would be inefficient
  QString cmd;
  bool wascmd;
  int count = 0;
  cConnPrefs *cp = sess->connection()->getConnPrefs ();
  for (unsigned int i = pos; i < command.length(); i++)
  {
    wascmd = true;
    switch (command[i].latin1()) {
      case 'n': cmd = ((cp == 0) ? QString("n") : cp->cmd(0)); break;
      case 'e': cmd = ((cp == 0) ? QString("e") : cp->cmd(2)); break;
      case 's': cmd = ((cp == 0) ? QString("s") : cp->cmd(4)); break;
      case 'w': cmd = ((cp == 0) ? QString("w") : cp->cmd(6)); break;
      case 'j': cmd = ((cp == 0) ? QString("ne") : cp->cmd(1)); break;
      case 'k': cmd = ((cp == 0) ? QString("se") : cp->cmd(3)); break;
      case 'l': cmd = ((cp == 0) ? QString("sw") : cp->cmd(5)); break;
      case 'm': cmd = ((cp == 0) ? QString("nw") : cp->cmd(7)); break;
      case 'u': cmd = ((cp == 0) ? QString("u") : cp->cmd(8)); break;
      case 'd': cmd = ((cp == 0) ? QString("d") : cp->cmd(9)); break;
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
        wascmd = false;
        count = count * 10 + (command[i].latin1() - '0');
        break;
      case ' ':
        //and spaces are ignored...
        wascmd = false;
        break;
    }
    if (wascmd)
    {
      if (count == 0) count = 1;
      for (int j = 0; j < count; j++)
        emit (newCommand (cmd));
      count = 0;
    }
  }
}

int cCmdProcessor::isFocusCommand (const QString &command)
{
  int pos;
  if ((focusstr.length() > 0) && (command.startsWith(focusstr))) {
    command.stripWhiteSpace(); //removes leading/trailing spaces
    if ((pos = command.find(focusstr,focusstr.length())) == -1)
      return -1; //return false if a second focustr can't be found
    return pos;
  }
  return -1; //should not get to here 
}

int cCmdProcessor::processFocusCommand (const QString &text, int pos)
{
  QString window, command;
  window = command = text;
      
  window.remove(pos,qstrlen(window));
  window.remove(0,focusstr.length()); //remove first focusstr
    
  command.remove(0,pos);
  command.remove(0,focusstr.length()); //remove next
    
  if ( (command.length() < 1) || (window == focusstr) || (window.length() < 1)) 
    return -1; //if nothing was passed after 2nd focusstr, return
  
  kmuddy->focusChange(window, command);

  return 0;
}

bool cCmdProcessor::isMacroCall (const QString &command,
    QString &mname, QString &params, bool isAliasTrigger)
{
  bool sc1 = false, sc2 = false;
  if ((scriptstr.length() > 0) && (command.startsWith (scriptstr)))
    sc1 = true;
  else
    if ((willExpandBS (isAliasTrigger)) && (command[0].latin1() == '\\') &&
        (command[1].latin1() == 'm'))
      sc2 = true;

  if (sc1 || sc2)
  {
    QString cmd = command;
    //remove the leading scriptstr or \m
    if (sc1)
      cmd.remove (0, scriptstr.length());
    else
      cmd.remove (0, 2);
    cmd = cmd.stripWhiteSpace();
    //first word
    mname = cmd.section (' ', 0, 0);
    //everything else
    params = cmd.section (' ', 1);
    params = params.stripWhiteSpace();
    return true;
  }
  else
    return false;
}

void cCmdProcessor::processMacroCall (const QString &mname,
      const QString &params, bool isAliasTrigger)
{
  cConnPrefs *cp = sess->connection()->getConnPrefs();
  if (cp == 0) // -> not a profile-based connection
  {
    //exclaim that macros are not available
    sess->console()->systemMessage (i18n ("Sorry, but macro calls are not available for quick connections."));
    return;
  }
  cVariableList *vl = cp->variableList();
  cGroupList *tgl = cp->triggerGroupList();
  cGroupList *agl = cp->aliasGroupList();

  //TODO - shall be done in a more flexible way - this kinda sucks...
  //known commands
  QString mnamel = mname.lower ();
  if (mnamel == "exec")
  {
    QString sname = params.section (' ', 0, 0, QString::SectionSkipEmpty);
    QString pars = params.section (' ', 1, -1, QString::SectionSkipEmpty);
    pars = vl->expandVariables (pars);
    runScript (sname, pars, isAliasTrigger);
  }
  else
  if (mnamel == "notify")
  {
    QString ip_port = params.section (' ', 0, 0, QString::SectionSkipEmpty);
    ip_port         = vl->expandVariables (ip_port);
    QString ip_data = params.section (' ', 1, -1, QString::SectionSkipEmpty);
    ip_data         = vl->expandVariables (ip_data);
    
    bool convert_ok;
    int  port;
    
    port = ip_port.toInt(&convert_ok);
    
    if (convert_ok)
    {
      // Send data to IP port
      doNotify(port,ip_data);
      
    } // endif converted IP port string to an int successfully
  }
  else
  if (mnamel == "get")
  {
    QString varname = params.section (' ', 0, 0, QString::SectionSkipEmpty);
    if (varname[0] == '$')
      varname = varname.mid (1);
    QString value = vl->getValue (varname);
    //inform the user
    if (value == QString::null)
      sess->console()->systemMessage ("$" + varname + " is not defined.");
    else
      sess->console()->systemMessage ("$" + varname + " = " + value);
  }
  else
  if (mnamel == "set")
  {
    QString varname = params.section (' ', 0, 0, QString::SectionSkipEmpty);
    QString value = params.section (' ', 1, -1, QString::SectionSkipEmpty);
    if (varname[0] == '$')
      varname = varname.mid (1);
    //no variable expansion for our parameter, to allow various tricks :)
    vl->set (varname, value);
    //we don't inform the user, because this can be called from scripts, and
    //we don't want to spam too much
    //TODO: make echoing variable changes configurable
  }
  else
  if (mnamel == "unset")
  {
    QString varname = params.section (' ', 0, 0, QString::SectionSkipEmpty);
    if (varname[0] == '$')
      varname = varname.mid (1);
    vl->unset (varname);
  }
  else
  if (mnamel == "inc")
  {
    QString varname = params.section (' ', 0, 0, QString::SectionSkipEmpty);
    QString value = params.section (' ', 1, 1, QString::SectionSkipEmpty);
    int val = value.toInt();
    if (val < 0)
      sess->console()->systemMessage ("Cannot increase by a negative value!");
    if (val == 0)
      sess->console()->systemMessage ("Increasing by 0 is useless...");
    if (varname[0] == '$')
      varname = varname.mid (1);
    vl->inc (varname, val);
  }
  else
  if (mnamel == "dec")
  {
    QString varname = params.section (' ', 0, 0, QString::SectionSkipEmpty);
    QString value = params.section (' ', 1, 1, QString::SectionSkipEmpty);
    int val = value.toInt();
    if (val < 0)
      sess->console()->systemMessage ("Cannot decrease by a negative value!");
    if (val == 0)
      sess->console()->systemMessage ("Decreasing by 0 is useless...");
    if (varname[0] == '$')
      varname = varname.mid (1);
    vl->dec (varname, val);
  }
  else
  if (mnamel == "provide-res")
  {
    QString varname = params.section (' ', 0, 0, QString::SectionSkipEmpty);
    vl->provideResource (varname);
  }
  else
  if (mnamel == "request-res")
  {
    QString varname = params.section (' ', 0, 0, QString::SectionSkipEmpty);
    bool ret = vl->requestResource (varname);
    if (!ret)
      sess->console()->systemMessage ("No free resources of that type.");
  }
  else
  if (mnamel == "tgroupon")
  {
    QString varname = params.stripWhiteSpace ();
    int id = tgl->getGroupId (varname);
    if (id != -1)
    {
      tgl->activateGroup (id, true);
      tgl->updateDialog ();
    }
  }
  else
  if (mnamel == "tgroupoff")
  {
    QString varname = params.stripWhiteSpace ();
    int id = tgl->getGroupId (varname);
    if (id != -1)
    {
      tgl->activateGroup (id, false);
      tgl->updateDialog ();
    }
  }
  else
  if (mnamel == "agroupon")
  {
    QString varname = params.stripWhiteSpace ();
    int id = agl->getGroupId (varname);
    if (id != -1)
    {
      agl->activateGroup (id, true);
      agl->updateDialog ();
    }
  }
  else
  if (mnamel == "agroupoff")
  {
    QString varname = params.stripWhiteSpace ();
    int id = agl->getGroupId (varname);
    if (id != -1)
    {
      agl->activateGroup (id, false);
      agl->updateDialog ();
    }
  }
  else
  if (mnamel == "echo")
  {
    if( !params.isEmpty() )
    {
      QColor c1, syscolor;
      sess->console()->getAllColors (c1, syscolor);
      QString txt = vl->expandVariables (params);
      cTextChunk *chunk = cTextChunk::makeLine (txt, syscolor,
          sess->ansiparser()->defaultBkColor(), sess->console());
      sess->console()->addLine (chunk);
      delete chunk;
    }
  }
  else
  if (mnamel == "tick")
  {
    cTimerTick *tt = sess->connection()->getConnPrefs()->timerList()->getTimerTick();
    QString sTickMessage;
    if( tt != NULL && tt->active() )
    {
      if( tt->countdown() > 1 )
        sTickMessage.sprintf( "Tick in %d seconds.", tt->countdown() );
      else if( tt->countdown() == 1 )
        sTickMessage.sprintf( "Tick in 1 second." );
      else if( tt->countdown() == 0 )
        sTickMessage.sprintf( "Tick now." );
      else
        sTickMessage.sprintf( "Resetting tick timer..." );
    }
    else
      sTickMessage = "Tick timer is not active.";
    sess->console()->systemMessage (sTickMessage);
  }
  else
  if (mnamel == "ticknow")
  {
    sess->connection()->getConnPrefs()->timerList()->getTimerTick()->resetTickCount();
  }
  else
  {
    //expand variables first
    QString pars = vl->expandVariables (params);
    //ask plug-ins if they know this macro
    if (!(pluginmanager->passMacro(sess, mname, pars)))
      //UNKNOWN COMMAND - assume it's an old-style script call
      //TODO: write some warning it a later version (probably in 0.8)
      //this may be removed in the future (but not before 1.0)
      runScript (mname, pars, isAliasTrigger);
  }
}


/** send information to external IP port */
void cCmdProcessor::doNotify(const int  port,
                             QString   &ip_data)
{
  list <cEventNotification *>::iterator ptr  = notifications.begin();
  list <cEventNotification *>::iterator last = notifications.end();
  
  while ((ptr                != last)&&
         ((*ptr)->port_num() != port)  )
  {
    ++ptr;
  } // endwhile more 'in progress' event notifications to search
  
  if (ptr == last)
  {
    cEventNotification *notify_event = new cEventNotification(port,ip_data);
          
    notifications.push_back(notify_event);
          
    connect(notify_event,
            SIGNAL (finished(cEventNotification *)),
            this,
            SLOT (portNotified(cEventNotification *)));
          
    notify_event->send();
    
  } // endif no other notifications in progress to this IP port at this time
  else
  {
    // Queue data for sending later
    (*ptr)->queue(ip_data);
    
  } // endelse need to queue notification
  
} // cCmdProcessor::doNotify


void cCmdProcessor::portNotified(cEventNotification *event_notify)
{
  list<cEventNotification *>::iterator ptr  = notifications.begin();
  list<cEventNotification *>::iterator last = notifications.end();
  
  while ((ptr  != last)        &&
         (*ptr != event_notify)  )
  {
    ++ptr;
  } // endwhile event notification not found
  
  if (ptr != last)
  {
    disconnect(*ptr,
               SIGNAL (finished(cEventNotification *)),
               this,
               SLOT (portNotified(cEventNotification *)));
    
    delete *ptr;
    
    notifications.erase(ptr);
  } // endif found relevant event notification
  
} // cCmdProcessor::portNotified


void cCmdProcessor::runScript (const QString &sname, const QString &params,
    bool isAliasTrigger)
{
  cConnPrefs *cp = sess->connection()->getConnPrefs();
  if (cp == 0) // -> not a profile-based connection
  //exclaim that macros are not available
    sess->console()->systemMessage (i18n ("Sorry, but scripts are not available for quick connections."));
  else
  {
    cScriptList *list = cp->scriptList();
    if (list == 0)    //no script list (strange)
      sess->console()->systemMessage (i18n ("Scripts are not available for unknown reason :-("));
    else
    {
      //Run it!!!  This will also display error message if something
      //goes wrong.
      QString pars = params;  //we need to modify that variable
      fixBackslashes (pars, isAliasTrigger);
      list->runScript (sname, pars);
    }
  }
}

void cCmdProcessor::processInput (const QString &text)
{
  doProcessInput (text, false);
}

void cCmdProcessor::processTriggerInput (const QString &text)
{
  doProcessInput (text, true);
}

void cCmdProcessor::doProcessInput (const QString &text, bool isAliasTrigger,
    bool expandAliases)
{
  //so we have the text...
  //if we don't parse, we just send it away...
  if (!parsing)
  {
    emit (newCommand (text));
    return; //there's nothing more left to do...
  }

  //look whether we should send the text as-is...
  if ((noparsestr.length() > 0) && (text.startsWith (noparsestr)))
  {
    QString t = text.right (text.length() - noparsestr.length());
    emit (newCommand (t));
    return; //there's nothing more left to do...
  }
  
  //First of all we split into commands divided by semicolons, \n chars and
  //separstr strings. We also trim some spaces.

  QString command;
  QString mytext = text;

  if (mytext.length() == 0) //single CR, nothing more...
    emit newCommand (mytext);
  
  bool focused=false;
  while (getNext (mytext, command, isAliasTrigger))
  {
    if (command.length() > 0)
    {
      //first look if there isn't a repeater sequence
      //if not, then repeater keeps original command and returns 1,
      //so we can handle both situations together :)
      unsigned int commandCount = repeater (command);

      //remember that command, we need to reset it in every iteration
      // (old code didn't support iterations and I'm too lazy to rewrite
      // it so that it doesn't change the command variable)
      QString thiscommand = command;
      //needed by script calls
      QString mname, params;
      //repeat this commandCount times...
      
      for (unsigned int cmdNo = 0; cmdNo < commandCount; cmdNo++)
      {
        command = thiscommand;
        int pos;
        //focusing command?
        if ((pos = isFocusCommand(command)) != -1) {
          focused=true;
          if (processFocusCommand(command,pos) == -1)
            focused=false;
        }
        else
        //look if it isn't a speed-walking command
        if ((pos = isSpeedWalkCommand (command)) != -1)
          processSpeedWalk (command, pos);
        else
        //look if it isn't a macro call
        if (isMacroCall (command, mname, params, isAliasTrigger))
          //macro calls must call variable expansion, if desired
          //this is to allow calls that need the variable names, not values
          // (/set and /unset, for instance)
          processMacroCall (mname, params, isAliasTrigger);
        else
        {
          //expand variables, if possible
          cConnPrefs *cp = sess->connection()->getConnPrefs();
          if (cp)
          {
            cVariableList *vl = cp->variableList();
            if (vl)
              command = vl->expandVariables (command);
          }                      
          //OK now go and fix backslashes (currently only \n was probably
          //expanded (only if expansion is enabled))
          fixBackslashes (command, isAliasTrigger);
          //if there are BSs in command and BS-expansion is alias-only, then
          //we must protect existing BSs from being expanded together with
          //those that came from the alias (yes I know it's ugly)
          protectBackslashes (command, isAliasTrigger);
          
          bool dontSendThisTime = false;
          //OK, now we got another command - check it against all aliases!
          if (expandAliases)
          {
            cConnPrefs *cp = sess->connection()->getConnPrefs ();
            //if cp is 0, we're not connected using a profile => no alias expansion
            if (cp != 0)
            {
              cAliasList *aliases = cp->aliasList();
              if (aliases != 0)
              {
                aliases->newString ();
                //match this command against aliases; also sends original command
                //if both original and aliased one have to be sent
                aliases->matchString (command);
                if (aliases->wasMatch ())
                {
                  dontSendThisTime = true;
                  QStringList cmds = aliases->getMatchedCommands ();
                  QStringList::iterator it;
                  for (it = cmds.begin(); it != cmds.end(); ++it)
                    //recursive call, but this time no alias expansion will occur
                    // (we don't want recursive aliases)
                    doProcessInput (*it, true, false);
                }
              }
            }
          }

          if (!dontSendThisTime)
          {
            //in case that there are protected BSs, unprotect them!
            unprotectBackslashes (command, isAliasTrigger);
            //this command should be sent - DO IT!
            
            //we don't want to send the actual <focustr>window<focustr>command
            //to the session... so this little hack works :p
            if (!focused) 
              emit (newCommand (command));
            else
              focused = false;  
          }
        }
      }
      command = "";
    }
    else //we have just an empty line; that's okay, we just send it
      emit (newCommand (command));    
  }
}



// cEventNotification implementation from here on

//***************************************************************
// Constructor - set up signals and initialise variables
//***************************************************************

cEventNotification::cEventNotification(const int      ip_port,
                                       const QString &ip_data)
{
  QString data;

  port = ip_port;
  data = ip_data+"\n";

  data_list.push_back(data);

  // Set up signal handling

  sock = new KExtendedSocket();

  connect(sock, SIGNAL(connectionSuccess()), this, SLOT(connected()));
  connect(sock, SIGNAL(connectionFailed(int)), this, SLOT(error()));
  connect(sock, SIGNAL(readyWrite()), this, SLOT(wroteAll()));
  connect(sock, SIGNAL(closed(int)), this, SLOT (connectionClosed()));

  sock->setSocketFlags(KExtendedSocket::inetSocket |
                       KExtendedSocket::bufferedSocket |
                       KExtendedSocket::streamSocket |
                       KExtendedSocket::noResolve);

} // cEventNotification::cEventNotification


//***************************************************************
// Destructor
//***************************************************************

cEventNotification::~cEventNotification()
{
   if (sock != NULL)
   {
      disconnect(sock, SIGNAL(connectionSuccess()), this, SLOT(connected()));
      disconnect(sock, SIGNAL(connectionFailed(int)), this, SLOT(error()));
      disconnect(sock, SIGNAL(readyWrite()), this, SLOT(wroteAll()));
      disconnect(sock, SIGNAL(closed(int)), this, SLOT (connectionClosed()));

      sock->cancelAsyncConnect();

      // delete socket
      sock->deleteLater();
      sock = NULL;
   } // endif socket active

} // cEventNotification::~cEventNotification


//***************************************************************
// Function to initiate information sending to the specified IP port
//***************************************************************

void cEventNotification::send()
{
   QString dest("127.0.0.1");

   sock->setAddress(dest,port);

   // Give up on connecting after 3 seconds
   sock->setTimeout(3);

   if (sock->startAsyncConnect() == -1)
   {
      cerr << "/notify error: Unable to connect\n";

      error();
   } // endif unable to commence asynchronous connect

} // cEventNotification::send


//***************************************************************
// Queue data for sending at next opportunity
//***************************************************************

void cEventNotification::queue(const QString &ip_data)
{
//  cerr << "Queueing to send: " << ip_data.latin1() << endl;
//  cerr.flush();

  QString data = ip_data+"\n";

  data_list.push_back(data);

   if (data_list.size() == 1)
   {
      list<QString>::iterator ptr = data_list.begin();

      // Note: this might not buffer the entire message contents.  Not completely sure
      int written = sock->writeBlock(ptr->latin1(),
                                     ptr->length());

      if (written != (int)ptr->length())
      {
         // NOTE: Far as I can tell, this never happens.  It's just here in case it ever does
         cerr << "/notify warning: writeBlock() was unable to buffer all data\n";
      } // endif unable to buffer all requested data

      if (written == -1)
      {
        // NOTE: This shouldn't ever happen because we wait until the buffer is empty before
        // trying to send any more data, but just in case...
        cerr << "/notify warning: writeBlock() buffer is full\n";
        data_list.clear();

        connectionClosed();
      } // endif unable to write data (write buffer is full)

      // turn on signal emission
      sock->enableWrite(true);
   } // endif queue was empty - need to restart sending

} // cEventNotification::queue


//***************************************************************
// Called when connected to IP port successfully
//***************************************************************

void cEventNotification::connected()
{
   list<QString>::iterator ptr = data_list.begin();

   // enable buffering
   sock->setBufferSize(-1,   // Unlimited read buffer size
                       -1);  // Unlimited write buffer size

   // Note: this might not buffer the entire message contents.  Not completely sure
   int written = sock->writeBlock(ptr->latin1(),
                                  ptr->length());

   if (written != (int)ptr->length())
   {
      // NOTE: Far as I can tell, this never happens.  It's just here in case it ever does
      cerr << "/notify warning: writeBlock() was unable to buffer all data\n";
   } // endif unable to buffer all requested data

   if (written == -1)
   {
     // NOTE: This shouldn't ever happen because we wait until the buffer is empty before
     // trying to send any more data, but just in case...
     cerr << "/notify warning: writeBlock() buffer is full\n";
     data_list.clear();

     connectionClosed();
   } // endif unable to write data (write buffer is full)

   // turn on signal emission
   sock->enableWrite(true);

} // cEventNotification::connected



//***************************************************************
// Wrote some data to the IP port
//***************************************************************

void cEventNotification::wroteAll()
{
   // At this point, the KExtendedSocket write buffer is empty

   data_list.pop_front();

   if (data_list.size() == 0)
   {
      // turn off signal emission
      sock->enableWrite(false);

      // TODO: Set up some kind of tick timer to close connection after a period of inactivity?
      // NOTE: doesn't seem to be necessary because the next attempt to send to an port that's been
      //       closed will give an error, thus causing a buffer flush and the socket to be closed.

   } // endif no more data to send at the moment
   else
   {
      list<QString>::iterator ptr = data_list.begin();

      int written = sock->writeBlock(ptr->latin1(),
                                     ptr->length());

      if (written != (int)ptr->length())
      {
         // NOTE: Far as I can tell, this never happens.  It's just here in case it ever does
         cerr << "/notify warning: writeBlock() was unable to buffer all data\n";
      } // endif unable to buffer all requested data

      if (written == -1)
      {
        // NOTE: This shouldn't ever happen because we wait until the buffer is empty before
        // trying to send any more data, but just in case...
        cerr << "/notify warning: writeBlock() buffer is full\n";
        data_list.clear();

        connectionClosed();
      } // endif unable to write data (write buffer is full)

   } // endelse more data to send

} // cEventNotification::wroteAll



//***************************************************************
// Connection closed - all done
//***************************************************************

void cEventNotification::connectionClosed()
{
   // Empty notification queue
   data_list.clear();

   // turn off signal emission
   sock->enableWrite(false);

   disconnect(sock, SIGNAL(connectionSuccess()), this, SLOT(connected()));
   disconnect(sock, SIGNAL(connectionFailed(int)), this, SLOT(error()));
   disconnect(sock, SIGNAL(readyWrite()), this, SLOT(wroteAll()));
   disconnect(sock, SIGNAL(closed(int)), this, SLOT (connectionClosed()));

   sock->cancelAsyncConnect();

   // delete socket
   sock->deleteLater();
   sock = NULL;

   if (port != -1)
   {
      port = -1;
      emit finished(this);
   } // endif not already announced we're finished

} // cEventNotification::connectionClosed


//***************************************************************
// An error occured
//***************************************************************

void cEventNotification::error()
{
   // Usually called when we attempt to connect to a port that's no longer
   // got a server attached to it.

   data_list.clear();

   if (port != -1)
   {
      port = -1;

      emit finished(this);

   } // endif not already announced we're finished

} // cEventNotification::error

