//
// C++ Implementation: cgauge
//
// Description: One gauge (no GUI)
//
//
// Author: Tomas Mecir <kmuddy@kmuddy.org>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "cgauge.h"

#include "cconnection.h"
#include "cconnprefs.h"
#include "cgaugebaritem.h"
#include "csession.h"
#include "cvariablelist.h"

#include <kconfig.h>

cGauge::cGauge (cSession *sess)
{
  varvalue = maxvarvalue = 0;
  _hidden = false;
  _color = white;
  
  gaugeitem = new cGaugeBarItem (this, (QWidget *) sess->gaugeBar());

  connect (this, SIGNAL (valueChanged ()), gaugeitem, SLOT (updateValue ()));

  s = sess;
}

cGauge::~cGauge()
{
  delete gaugeitem;
}

cSaveableField *cGauge::newInstance ()
{
  return new cGauge (s);
}

void cGauge::setVariable (const QString &val)
{
  _variable = val;
  varvalue = 0;
  emit valueChanged ();
}

void cGauge::setMaxVariable (const QString &val)
{
  _maxvariable = val;
  maxvarvalue = 0;
  emit valueChanged ();
}

void cGauge::setCaption (const QString &val)
{
  _caption = val;
  emit valueChanged ();
}

void cGauge::setColor (const QColor &c)
{
  _color = c;
  emit valueChanged ();
}

void cGauge::setHidden (bool h)
{
  bool oldh = _hidden;
  _hidden = h;
  if (h != oldh)
    emit valueChanged ();
}

void cGauge::varValueChanged (const QString &var, int newval)
{
  if ((var == _variable) && (varvalue != newval))
  {
    varvalue = newval;
    emit valueChanged ();
  }
  if ((var == _maxvariable) && (maxvarvalue != newval))
  {
    maxvarvalue = newval;
    emit valueChanged ();
  }
}

void cGauge::recalc ()
{
  cVariableList *vars = s->connection()->getConnPrefs()->variableList();
  if (_variable)
    varValueChanged (_variable, vars->getIntValue(_variable));
  if (_maxvariable)
    varValueChanged (_maxvariable, vars->getIntValue(_maxvariable));
}

void cGauge::load (KConfig *config, const QString &group)
{
  config->setGroup (group);

  setVariable (config->readEntry ("Variable", QString::null));
  setMaxVariable (config->readEntry ("Max variable", QString::null));
  setCaption (config->readEntry ("Caption", QString::null));
  QColor clr = Qt::white;
  setColor (config->readColorEntry ("Color", &clr));
  setHidden (config->readBoolEntry ("Hidden", false));

  recalc ();
}

void cGauge::save (KConfig *config, const QString &group)
{
  config->setGroup (group);

  config->writeEntry ("Variable", _variable);
  config->writeEntry ("Max variable", _maxvariable);
  config->writeEntry ("Caption", _caption);
  config->writeEntry ("Color", _color);
  config->writeEntry ("Hidden", _hidden);
}


#include "cgauge.moc"
