/***************************************************************************
                          ctextprocessor.h  -  text processor
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Po oct 14 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CTEXTPROCESSOR_H
#define CTEXTPROCESSOR_H

#include <qobject.h>
#include <qcolor.h>

#include <list>

#include "csessionbase.h"
#include "ctextchunk.h"

using namespace std;

class cTextChunk;

class QTimer;

/**
Text processor - handles trigger execution and such...
  *@author Tomas Mecir
  */

class cTextProcessor : public QObject, public cSessionBase {
   Q_OBJECT
public: 
  cTextProcessor ();
  ~cTextProcessor ();
  int linesReceived () { return lines; };

  /** recolorize the line - used by color triggers */
  void recolorize (list<colorChange> &changes);
  /** this line should be gagged - called by gagging triggers */
  void gagLine () { gag = true; };
  /** this line should be gagged in primary window */
  void gagLineInPrimary () { primarygag = true; };
  void setHavePrompt () { haveprompt = true; };
  /** handle some issued when sending a command */
  void sendingCommand ();

  void setOutputWindow(QString winname) { wname = winname; };
public slots:
  /** preprocess this text */
  void preprocess (const QString &text);
  /** process output from preprocessor script */
  void preprocessorOutput (const QString &text);

  /** parse this text, pass it to MXP parser if needed and so... */
  void parseText (const QString &text);
  
  // processing of various received items...
  
  /** process this text */
  void gotNewText (const QString &text);
  /** process fg color */
  void gotFgColor (QColor color);
  /** process bg color */
  void gotBgColor (QColor color);
  /** process attribute */
  void gotAttrib (int a);
  /** process A-link */
  void gotALink (const QString &name, const QString &url, const QString &text,
      const QString &hint);
  /** process SEND-link */
  void gotSENDLink (const QString &name, const QString &command, const QString &text,
      const QString &hint, bool toprompt, bool ismenu);
  /** process newline */
  void gotNewLine ();

    
  void flush ();
  void receivedGA ();
  /** used to display partial lines in the status bar */
  void moreTextHere ();
signals:
  /** text preprocessed, can continue processing... */
  void preprocessed (const QString &text);
  
  /** we got this plain text - ANSI parser should be connected here */
  void plainText (const QString &text);
  
  void displayLine (cTextChunk *chunk);
  void displayPrompt (cTextChunk *chunk);
  void textLine (const QString &text);
  void receivedPrompt (const QString &prompt);
  void showPartialLine (const QString &line);
protected slots:
  /** used by prompt-detection timer */
  void timeout ();
protected:
  /** create a new chunk */
  void createChunk ();

  /** a timer used to auto-detect prompts */
  QTimer *pdtimer;
  int elapsedticks;

  cTextChunk *chunk;
  
  int lines;

  /** gagging */
  bool gag, primarygag;
  /** prompt detection */
  bool haveprompt, gotprompt;

  bool owin;
  QString wname;
  
  //current text attributes, used to set startpos correctly when breaking lines
  QColor fg, bg;
  int attrib;
};

#endif
